@@ -12,7 +12,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Circular array with variable storage capacity.
+"""
+Variable Storage Capacity
+=========================
+
+**Circular array with variable storage capacity.**
 
 - O(1) pops either end 
 - O(1) amortized pushes either end 
@@ -21,36 +25,26 @@
 - iterable, can safely mutate while iterators continue iterating over previous state
 - comparisons compare identity before equality, like builtins
 - in boolean context, falsy when empty, otherwise truthy
-- factory function ca produces a variable storage capacity circular array from its arguments
+- factory function ``ca`` produces a variable storage capacity circular array from its arguments
 
 """
 from collections.abc import Callable, Iterable, Iterator
-from typing import cast, Never, overload, TypeVar
+from typing import cast, overload
 
 __all__ = ['CA', 'ca']
 
-I = TypeVar('I')
-T = TypeVar('T')
-
 
 class CA[I]():
 
-    L = TypeVar('L')
-    R = TypeVar('R')
-    U = TypeVar('U')
-
     __slots__ = '_items', '_cnt', '_cap', '_front', '_rear'
 
     def __init__(
             self,
             items: Iterable[I] | None = None
         ) -> None:
-        """Basically a list that can be grown from both ends
-        in O(1) time and space complexity.
-
-        :param items: optional iterable to initial populate circular array
-        :raises TypeError: if items is not Iterable
-
+        """
+        :param items: Optional iterable to initial populate circular array.
+        :raises TypeError: When ``items`` not Iterable.
         """
         if items is None:
             self._items: list[I | None] = [None, None]
@@ -312,10 +306,9 @@ class CA[I]():
         return True
 
     def pushl(self, *items: I) -> None:
-        """Push items onto left side (front) of circular array.
-
-        :param items: items pushed onto circular array from left
+        """Push ``items`` on from left.
 
+        :param items: Items pushed onto circular array from left (front).
         """
         for item in items:
             if self._cnt == self._cap:
@@ -331,10 +324,9 @@ class CA[I]():
                 )
 
     def pushr(self, *items: I) -> None:
-        """Push items onto right side (rear) of circular array.
-
-        :param items: items pushed onto circular array from right
+        """Push ``items`` on from right.
 
+        :param items: Items pushed onto circular array from right (rear).
         """
         for item in items:
             if self._cnt == self._cap:
@@ -349,12 +341,11 @@ class CA[I]():
                     self._cnt + 1,
                 )
 
-    def popl(self) -> I | Never:
-        """Pop item off left side (front) of circular array.
-
-        :return: item popped from left side of circular array
-        :raises ValueError: when called on an empty circular array
+    def popl(self) -> I:
+        """Pop single item off from left side.
 
+        :returns: Item popped from left side (front) of circular array.
+        :raises ValueError: When called on an empty circular array.
         """
         if self._cnt > 1:
             (
@@ -387,12 +378,11 @@ class CA[I]():
             raise ValueError(msg)
         return cast(I, d)
 
-    def popr(self) -> I | Never:
-        """Pop item off right side (rear) of circular array.
-
-        :return: item popped from right side of circular array
-        :raises ValueError: when called on an empty circular array
+    def popr(self) -> I:
+        """Pop single item off from right.
 
+        :returns: Item popped from right side (rear) of circular array.
+        :raises ValueError: When called on an empty circular array.
         """
         if self._cnt > 1:
             (
@@ -427,11 +417,10 @@ class CA[I]():
 
     def popld(self, default: I) -> I:
         """Pop one item from left side of the circular array, provide
-        a mandatory default value. "Safe" version of popl.
-
-        :param default: item returned if circular array is empty
-        :return: item popped from left side or default item if empty
+        a mandatory default value. "Safe" version of ``popl``.
 
+        :param default: Item returned if circular array is empty.
+        :returns: Item popped from left side or default item if empty.
         """
         try:
             return self.popl()
@@ -440,11 +429,10 @@ class CA[I]():
 
     def poprd(self, default: I) -> I:
         """Pop one item from right side of the circular array, provide
-        a mandatory default value. "Safe" version of popr.
-
-        :param default: item returned if circular array is empty
-        :return: item popped from right side or default item if empty
+        a mandatory default value. "Safe" version of ``popr``.
 
+        :param default: Item returned if circular array is empty.
+        :returns: Item popped from right side or default item if empty.
         """
         try:
             return self.popr()
@@ -454,9 +442,8 @@ class CA[I]():
     def poplt(self, maximum: int) -> tuple[I, ...]:
         """Pop multiple items from left side of circular array.
 
-        :param maximum: maximum number of items to pop, may pop less if not enough items
-        :return: items in the order popped, left to right
-
+        :param maximum: Maximum number of items to pop, may pop less if not enough items.
+        :returns: Tuple of items in the order popped, left to right.
         """
         item_list: list[I] = []
 
@@ -473,9 +460,8 @@ class CA[I]():
     def poprt(self, maximum: int) -> tuple[I, ...]:
         """Pop multiple items from right side of circular array.
 
-        :param maximum: maximum number of items to pop, may pop less if not enough items
-        :return: items in the order popped, right to left
-
+        :param maximum: Maximum number of items to pop, may pop less if not enough items.
+        :returns: Tuple of items in the order popped, right to left.
         """
         item_list: list[I] = []
         while maximum > 0:
@@ -488,10 +474,9 @@ class CA[I]():
         return tuple(item_list)
 
     def rotl(self, n: int = 1) -> None:
-        """Rotate items left.
-
-        :param n: number of times to shift elements to the left
+        """Rotate items to the left.
 
+        :param n: Number of times to shift elements to the left.
         """
         if self._cnt < 2:
             return
@@ -499,10 +484,9 @@ class CA[I]():
             self.pushr(self.popl())
 
     def rotr(self, n: int = 1) -> None:
-        """Rotate items right.
-
-        :param n: number of times to shift elements to the right
+        """Rotate items to the right.
 
+        :param n: Number of times to shift elements to the right.
         """
         if self._cnt < 2:
             return
@@ -510,22 +494,20 @@ class CA[I]():
             self.pushl(self.popr())
 
     def map[U](self, f: Callable[[I], U]) -> "CA[U]":
-        """Apply function f over the circular array's contents,
-
-        :param f: callable from type I to type U
-        :return: new circular array instance
+        """Apply function ``f`` over the circular array's contents.
 
+        :param f: Callable from type ``I`` to type ``U``.
+        :returns: New auto-resizing circular array instance.
         """
         return CA(map(f, self))
 
-    def foldl[L](self, f: Callable[[L, I], L], start: L | None = None) -> L | Never:
+    def foldl[L](self, f: Callable[[L, I], L], start: L | None = None) -> L:
         """Fold left with a function and optional starting item.
 
-        :param f: first argument to f is for the accumulator
-        :param start: optional starting item
-        :return: reduced value produced by the left fold
-        :raises ValueError: when circular array empty and no starting item given
-
+        :param f: Folding function, first argument to ``f`` is for the accumulator.
+        :param start: Optional starting item.
+        :returns: Reduced value produced by the left fold.
+        :raises ValueError: When circular array empty and no starting item given.
         """
         if self._cnt == 0:
             if start is None:
@@ -544,14 +526,13 @@ class CA[I]():
             acc = f(acc, d)
         return acc
 
-    def foldr[R](self, f: Callable[[I, R], R], start: R | None = None) -> R | Never:
+    def foldr[R](self, f: Callable[[I, R], R], start: R | None = None) -> R:
         """Fold right with a function and an optional starting item.
 
-        :param f: second argument to f is for the accumulator
-        :param start: optional starting item
-        :return: reduced value produced by the right fold
-        :raises ValueError: when circular array empty and no starting item given
-
+        :param f: Folding function, second argument to ``f`` is for the accumulator.
+        :param start: Optional starting item.
+        :returns: Reduced value produced by the right fold.
+        :raises ValueError: When circular array empty and no starting item given.
         """
         if self._cnt == 0:
             if start is None:
@@ -573,8 +554,7 @@ class CA[I]():
     def capacity(self) -> int:
         """Return current storage capacity of the circular array.
 
-        :return: current storage capacity
-
+        :returns: Current storage capacity.
         """
         return self._cap
 
@@ -595,8 +575,7 @@ class CA[I]():
     def fraction_filled(self) -> float:
         """Find fraction of the storage capacity which is filled.
 
-        :return: the ratio count/capacity
-
+        :returns: The ratio count/capacity.
         """
         return self._cnt / self._cap
 
@@ -605,8 +584,7 @@ class CA[I]():
         storage capacity. To just compact the circular array, do not
         provide ``minimum_capacity``.
 
-        :param minimum_capacity: minimum storage capacity to compact the circular array
-
+        :param minimum_capacity: Minimum storage capacity to compact the circular array.
         """
         self._compact_storage_capacity()
         if (min_cap := minimum_capacity) > self._cap:
@@ -624,8 +602,7 @@ class CA[I]():
 def ca[T](*items: T) -> CA[T]:
     """Produce circular array from a variable number of arguments.
 
-    :param items: initial items for a new circular array
-    :return: new variable storage capacity circular array
-
+    :param items: Initial items for a new auto-resizing circular array.
+    :returns: New variable storage capacity circular array.
     """
     return CA(items)
